"""
This module contains the GUI components for the AxoDen application.

Start the tkinter GUI by running the following command:

.. code-block:: bash

    python -m axoden.gui.gui_projections_quantification

For the streamlit app, run the following command:

.. code-block:: bash

    python -m axoden.gui.streamlit_app
"""
